##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: __init__.py 81270 2007-10-31 14:02:39Z jukart $
"""
__docformat__ = "reStructuredText"

from StringIO import StringIO
import config

if config.PREFER_Z3C_PT:
    from z3c.pt.pagetemplate import ViewPageTemplateFile
    from z3c.pt.pagetemplate import PageTemplateFile

    def bind_template(pt, view):
        return pt.bind(view)

    def bind_macro(template, view, request, macro):
        return template.bind(
            view, request=request, macro=macro)
    
else:
    from zope.tal.talinterpreter import TALInterpreter
    from zope.pagetemplate.pagetemplatefile import PageTemplateFile
    from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile    
    from zope.app.pagetemplate.viewpagetemplatefile import BoundPageTemplate as \
         bind_template

    def bind_macro(template, view, request, macro):
        program = template.macros[macro]

        def render(content_type=None, **kwargs):
            output = StringIO(u'')
            namespace = template.pt_getContext(
                view, request, options=kwargs)
            context = template.pt_getEngineContext(namespace)
            TALInterpreter(program, None,
                           context, output, tal=True, showtal=False,
                           strictinsert=0, sourceAnnotations=False)()
            if not request.response.getHeader("Content-Type"):
                request.response.setHeader(
                    "Content-Type", content_type)
            return output.getvalue()

        return render
    
class ViewPageTemplateFile(ViewPageTemplateFile):
    """View page template file."""



#!/usr/bin/python2.5

import os
import re
import sys
import difflib
import optparse
import subprocess

parser = optparse.OptionParser()
parser.add_option("-n", "--dry-run",
                  action="store_true", dest="dry_run", default=False,
                  help="Don't actually make any changes.")
parser.add_option("-v", "--verbose",
                  action="store_true", dest="verbose", default=False,
                  help="Verbose output.")

re_vptf_sub = (
    re.compile(
    r'^from zope\.app\.pagetemplate(\.viewpagetemplatefile)? import ViewPageTemplateFile$',
    re.M),
    r'from z3c.ptcompat import ViewPageTemplateFile')

re_ptf_sub = (
    re.compile(
    r'^from zope\.pagetemplate(\.pagetemplatefile)? import PageTemplateFile$',
    re.M),
    r'from z3c.ptcompat import PageTemplateFile')

def log(msg):
    for line in msg.split('\n'):
        print ">>> %s" % line

def status(msg):
    for line in msg.split('\n'):
        print "    %s" % line

def main(options, args):
    if options.dry_run:
        status("Warning: Dry run---no changes will be made to the file-system.")

    path = os.getcwd()
    status("Working directory: %s" % path)

    log("Analyzing source-code...")
    registry = {}
    count = 0
    for arg, dirname, names in os.walk(path):
        for name in names:
            base, ext = os.path.splitext(name)
            if ext != '.py':
                continue

            if options.verbose and count > 0 and count % 500 == 0:
                status("Processed %d files." % count)

            count += 1
            filename = os.path.join(arg, name)

            diff = create_diff(filename)
            if diff is not None:
                registry[filename] = diff

    status("%d files processed" % count)
    log("Updating import-statements in %d files..." % len(registry.keys()))

    if options.dry_run:
        queue = ['patch', '-p0', '--dry-run'],
    else:
        queue = ['patch', '-p0', '--dry-run'], ['patch', '-p0']

    for args in queue:
        if options.verbose:
            args.append('--verbose')
        else:
            args.append('--quiet')

        proc = subprocess.Popen(
            " ".join(args),
            shell=True,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            )

        diffs = []
        for filename, body in registry.iteritems():
            if options.verbose:
                changes = body.count('+++ ')
                status("%s (%d %s)." % (
                    filename, changes, 'changes' if (changes > 1) else 'change'))

            diffs.append(body)

        if not diffs:
            break

        output, err = proc.communicate("\n".join(diffs)+'\n')
        
        if proc.returncode != 0:
            map(status, output.split('\n'))
            status("An error occurred while applying patches.")
            break
    else:
        status("Succesfully patched files.")
        
    if '--dry-run' in args:
        status("No files were changed.")
    
    sys.exit(proc.returncode)

def create_diff(filename):
    original = open(filename).read()
    new = original

    # process regular expression substitutions
    for exp, replacement in (re_vptf_sub, re_ptf_sub):
        new = exp.sub(replacement, new)
            
    if original != new:
        return "\n".join(map(str.rstrip, difflib.unified_diff(
            original.split('\n'),
            new.split('\n'),
            filename,
            filename)))
        
if __name__ == "__main__":
    main(*parser.parse_args())

    



"""Loading this module will monkey-patch ZPT template classes such
that they render using Chameleon.

Since many templates are instantiated at module-import, we patch using
a duck-typing strategy.

We replace the ``__get__``-method of the ViewPageTemplateFile
class. This allows us to return a Chameleon template instance,
transparent to the calling class.
"""

from zope.app.pagetemplate.viewpagetemplatefile import BoundPageTemplate
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile as \
     ZopeViewPageTemplateFile

from z3c.pt.pagetemplate import ViewPageTemplateFile

_marker = object()

def get_bound_template(self, instance, type):
    if instance is None:
        return self
        
    template = getattr(self, '_template', _marker)
    if template is _marker:
        self._template = template = ViewPageTemplateFile(self.filename)

    return template.bind(instance)

ZopeViewPageTemplateFile.__get__ = get_bound_template



import os
import sys
from new import function

from zope.interface import classImplements
from zope.viewlet import viewlet
from zope.viewlet import manager
from zope.viewlet import metaconfigure as viewletmeta
from zope.viewlet.interfaces import IViewletManager
from zope.app.publisher.browser import viewmeta
from zope.app.pagetemplate import simpleviewclass

from z3c.ptcompat import ViewPageTemplateFile
from z3c.ptcompat import config

def package_home(gdict):
    filename = gdict["__file__"]
    return os.path.dirname(filename)

def clone_and_replace_globals(func, new_globals):
    func_globals = func.func_globals.copy()
    func_globals.update(new_globals)
    func_defaults = func.func_defaults or ()
    new_func = function(func.func_code, func_globals,
                        func.func_name, func_defaults)
    new_func.__doc__ = getattr(func, '__doc__', '')
    func_dict = getattr(func, '__dict__', None)
    if func_dict is not None:
        new_func.__dict__ = func_dict.copy()
    return new_func

def SimpleViewClass(src, offering=None, used_for=None, bases=(), name=u''):
    if offering is None:
        offering = sys._getframe(1).f_globals

    if isinstance(offering, dict):
        offering = package_home(offering)

    bases += (simpleviewclass.simple, )

    class_ = type("SimpleViewClass from %s" % src, bases,
                  {'index': ViewPageTemplateFile(src, offering),
                   '__name__': name})

    if used_for is not None:
        class_.__used_for__ = used_for

    return class_

def SimpleViewletClass(src, offering=None, bases=(),
                       attributes=None, name=u''):
    if offering is None:
        offering = sys._getframe(1).f_globals

    if isinstance(offering, dict):
        offering = package_home(offering)

    # Create the base class hierarchy
    bases += (viewlet.simple, viewlet.ViewletBase)

    attrs = {'index' : ViewPageTemplateFile(src, offering),
             '__name__' : name}
    if attributes:
        attrs.update(attributes)

    # Generate a derived view class.
    class_ = type("SimpleViewletClass from %s" % src, bases, attrs)

    return class_

def ViewletManager(name, interface, template=None, bases=()):
    attrs = {'__name__' : name}
    if template is not None:
        attrs['template'] = ViewPageTemplateFile(template)

    if manager.ViewletManagerBase not in bases:
        # Make sure that we do not get a default viewlet manager mixin, if the
        # provided base is already a full viewlet manager implementation.
        if not (len(bases) == 1 and
                IViewletManager.implementedBy(bases[0])):
            bases = bases + (manager.ViewletManagerBase,)

    ViewletManager = type(
        '<ViewletManager providing %s>' % interface.getName(), bases, attrs)
    classImplements(ViewletManager, interface)
    return ViewletManager

def page_directive(_context, name, *args, **kwargs):
    class_ = kwargs.get('class_')
    template = kwargs.get('template')

    if template:
        bases = class_ and (class_,) or ()
        kwargs['class_'] = SimpleViewClass(str(template), bases=bases,
                                           name=name)
        del kwargs['template']

    return viewmeta.page(_context, name, *args, **kwargs)

new_page_globals = dict(page=page_directive)
class pages_directive(viewmeta.pages):

    page = clone_and_replace_globals(viewmeta.pages.page.im_func,
                                     new_page_globals)

def viewlet_directive(_context, name, *args, **kwargs):
    class_ = kwargs.get('class_')
    template = kwargs.get('template')

    if template:
        bases = class_ and (class_,) or ()
        kwargs['class_'] = SimpleViewletClass(str(template),
                                              bases=bases, name=name)
        del kwargs['template']

    return viewletmeta.viewletDirective(_context, name, *args, **kwargs)

def viewlet_manager_directive(_context, name, *args, **kwargs):
    class_ = kwargs.get('class_')
    template = kwargs.get('template')
    provides = kwargs.setdefault('provides', IViewletManager)

    if template:
        bases = class_ and (class_,) or ()
        kwargs['class_'] = ViewletManager(
            name, provides, template=str(template), bases=bases)
        del kwargs['template']

    return viewletmeta.viewletManagerDirective(_context, name, *args, **kwargs)

try:
    # Make zope.app.form a soft-dependency. We only register custom
    # directives if it is available.
    from zope.app.form.browser import metaconfigure as formmeta
except ImportError:
    pass
else:
    if config.PREFER_Z3C_PT:
        # Replace globals in *Factory by the ones from our package, cloning
        # the existing functions so we don't have to re-define them.
        new_factory_globals = dict(ViewPageTemplateFile=ViewPageTemplateFile,
                                   SimpleViewClass=SimpleViewClass)

        AddViewFactory = clone_and_replace_globals(formmeta.AddViewFactory,
                                                   new_factory_globals)
        EditViewFactory = clone_and_replace_globals(formmeta.EditViewFactory,
                                                    new_factory_globals)
        DisplayViewFactory = clone_and_replace_globals(formmeta.DisplayViewFactory,
                                                       new_factory_globals)

        # Now, replace globals in the directive handlers' __call__ by our own
        # factories that were cloned right above.
        new_form_globals = dict(AddViewFactory=AddViewFactory,
                                EditViewFactory=EditViewFactory,
                                DisplayViewFactory=DisplayViewFactory)

        class AddFormDirective(formmeta.AddFormDirective):
            __call__ = clone_and_replace_globals(
                formmeta.AddFormDirective.__call__.im_func,
                new_form_globals)

        class EditFormDirective(formmeta.EditFormDirective):
            __call__ = clone_and_replace_globals(
                formmeta.EditFormDirective.__call__.im_func,
                new_form_globals)

        class FormDirective(formmeta.FormDirective):
            __call__ = clone_and_replace_globals(
                formmeta.FormDirective.__call__.im_func,
                new_form_globals)

        class SubeditFormDirective(formmeta.SubeditFormDirective):
            __call__ = clone_and_replace_globals(
                formmeta.SubeditFormDirective.__call__.im_func,
                new_form_globals)

        class SchemaDisplayDirective(formmeta.SchemaDisplayDirective):
            __call__ = clone_and_replace_globals(
                formmeta.SchemaDisplayDirective.__call__.im_func,
                new_form_globals)
    else:
        AddViewFactory = formmeta.AddViewFactory
        EditViewFactory = formmeta.EditViewFactory
        DisplayViewFactory = formmeta.DisplayViewFactory

        AddFormDirective = formmeta.AddFormDirective
        EditFormDirective = formmeta.EditFormDirective
        FormDirective = formmeta.FormDirective
        SubeditFormDirective = formmeta.SubeditFormDirective
        SchemaDisplayDirective = formmeta.SchemaDisplayDirective



from setuptools import setup, find_packages
import sys, os


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()


version = '0.5.4'
tests_require = ['z3c.pt',
                 'zope.tal',
                 'zope.viewlet',
                 'zope.app.form',
                 'zope.app.publisher',
                 'zope.app.pagetemplate',
                 'lxml',
                 ],


setup(name='z3c.ptcompat',
      version=version,
      description="Compatibility-layer for Zope Page Template engines.",
      long_description=(
        ".. contents::\n\n" +
        read('README.txt')
        + "\n\n" +
        read('src', 'z3c', 'ptcompat', 'zcml.txt')
        + "\n\n" +
        read("CHANGES.txt")
        ),
      classifiers=[
        "Framework :: Plone",
        "Framework :: Zope2",
        "Framework :: Zope3",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='zpt template zope',
      url='http://pypi.python.org/pypi/z3c.ptcompat',
      author='Zope Corporation and Contributors',
      author_email='zope-dev@zope.org',
      license='ZPL',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['z3c'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          ],
      extras_require = dict(
        zpt = ['zope.app.pagetemplate', 'zope.tal'],
        z3cpt = ['z3c.pt'],
        test = tests_require, # used by buildout.cfg testrunner
        ),
      tests_require = tests_require,
      test_suite="z3c.ptcompat.tests.test_doctests.test_suite",
      )



import zope.interface
import zope.component

import os
import unittest
import doctest

OPTIONFLAGS = (doctest.ELLIPSIS |
               doctest.NORMALIZE_WHITESPACE)

import zope.component.testing
import zope.configuration.xmlconfig

import z3c.pt
import z3c.ptcompat

class TestParticipation(object):
    principal = 'foobar'
    interaction = None

def setUp(test):
    zope.component.testing.setUp(test)
    zope.configuration.xmlconfig.XMLConfig('meta.zcml', z3c.ptcompat)()
    zope.configuration.xmlconfig.XMLConfig('configure.zcml', z3c.pt)()

def tearDown(test):
    zope.component.testing.tearDown(test)

def test_suite():
    import z3c.ptcompat.tests
    path = z3c.ptcompat.tests.__path__[0]

    globs = dict(
        os=os,
        path=path,
        interface=zope.interface,
        component=zope.component)
    
    return unittest.TestSuite([
        doctest.DocFileSuite(
        "zcml.txt",
        optionflags=OPTIONFLAGS,
        globs=globs,
        setUp=setUp,
        tearDown=tearDown,
        package="z3c.ptcompat")])

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')
